#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wyrd', 'wyrd.constrained_types', 'wyrd.read_once']

package_data = \
{'': ['*']}

extras_require = \
{":python_version == '3.6' or python_version == '3.7'": ['typing-extensions']}

setup(name='wyrd',
      version='0.5.0',
      description='Library for helpers with Domain driven security',
      author='meadsteve',
      author_email='meadsteve@gmail.com',
      url='https://github.com/meadsteve/constrained_types/',
      packages=packages,
      package_data=package_data,
      extras_require=extras_require,
      python_requires='~=3.6',
     )
